#!/usr/bin/python
# -*- coding: utf-8 -*-

from ...num import bitconverter, byteorder

BYTEORDER_HOST    = 0x0000
BYTEORDER_NETWORK = 0x0001


###############################################################################
class ListDataRaw(object):

	TDC_UNIT   = 0.0		#unit of 'tdc'    --> nanoseconds
	TDCFP_UNIT = 0.0		#unit of 'tdc_fp' --> picoseconds
	EVENTDATA_BYTES = 0		#list data block size

	############################################
	def __init__(self, data = None, data_byteorder = BYTEORDER_NETWORK):
		self.__initializing()
		if data is not None: self.adjust(data, data_byteorder)

	############################################
	def __initializing(self):
		self.mTdc = 0
		self.mTdcFp = 0
		self.mChannel = 0
		self.mQDC = 0
		self.mNanoseconds = 0.0
		#####
		self.mTotal = 0
		self.mFall  = 0
		self.mRise  = 0

	############################################
	def adjust(self, data, data_byteorder = BYTEORDER_NETWORK):
		pass

	############################################
	def getTdc2Picoseconds(self):
		value = self.mNanoseconds * 1000.0
		return value

	############################################
	def getTdc2Microseconds(self):
		value = self.mNanoseconds / 1000.0
		return value

	############################################
	def getTdc2Milliseconds(self):
		value = self.mNanoseconds / 1000000.0
		return value
	
	############################################
	def getTdc2Seconds(self):
		value = self.mNanoseconds / 1000000000.0
	
	############################################
	@classmethod
	def calcTdc2Nanoseconds(cls, tdc, tdcfp):
		nsec = float(tdc)   * cls.TDC_UNIT
		psec = float(tdcfp) * cls.TDCFP_UNIT
		return (nsec + (psec / 1000.0))
	
	############################################
	@classmethod
	def generateListdata(cls, data, data_byteorder = BYTEORDER_NETWORK):

		listData = []
		blocks = int(len(data) / cls.EVENTDATA_BYTES)

		for i in range(blocks):
			index = i * cls.EVENTDATA_BYTES
			list0 = cls(data[index:index+cls.EVENTDATA_BYTES], data_byteorder)
			listData.append(list0)
		
		return listData

###############################################################################
###############################################################################
###############################################################################
class ListData10(ListDataRaw):

	TDC_UNIT   = 2.0		#unit of 'tdc'    --> nanoseconds
	TDCFP_UNIT = 7.8125		#unit of 'tdc_fp' --> picoseconds
	EVENTDATA_BYTES = 10	#list data block size

	############################################
	"""
	def __init__(self, data = None, data_byteorder = BYTEORDER_NETWORK):
		self._ListDataRaw__initializing()

		if data is not None:
			self.adjust(data, data_byteorder)
	"""

	############################################
	def adjust(self, data, data_byteorder = BYTEORDER_NETWORK):
		
		tdc = bitconverter.toIntU64(data[0:8])
		qdc = bitconverter.toIntU16(data[8:10])

		#byteorder change
		if data_byteorder != BYTEORDER_HOST:
			tdc = byteorder.hton64(tdc)
			qdc = byteorder.htons(qdc)
		
		self.mTdc = (tdc >> 8) & 0x00ffffffffffffff
		self.mTdcFp = tdc & 0x00ff
		self.mChannel = ((qdc >> 13) & 0x07) + 1
		self.mQDC = qdc & 0x1fff
		self.mNanoseconds = self.calcTdc2Nanoseconds(self.mTdc, self.mTdcFp)
		return


###############################################################################
###############################################################################
###############################################################################
class ListData16(ListData10):

	EVENTDATA_BYTES = 16	#list data block size

	############################################
	"""
	def __init__(self, data = None, data_byteorder = BYTEORDER_NETWORK):
		self.__initializing()

		if data is not None:
			self.adjust(data, data_byteorder)
	"""

	############################################
	def adjust(self, data, data_byteorder = BYTEORDER_NETWORK):
		super(ListData16, self).adjust(data[6:16], data_byteorder)

		self.mTotal = bitconverter.toIntU16(data[0:2])
		self.mFall  = bitconverter.toIntU16(data[2:4])
		self.mRise  = bitconverter.toIntU16(data[4:6])
	
		#byteorder change
		if data_byteorder != BYTEORDER_HOST:
			self.mTotal = byteorder.htons(self.mTotal)
			self.mFall = byteorder.htons(self.mFall)
			self.mRise = byteorder.htons(self.mRise)
		
		return

